<?php

/*
 * LICENSE
 * 1. If you like to use this class for personal purposes, it's free.
 * 2. For comercial purposes, please contact me (http://maettig.com/email).
 *    I'll send a license to you.
 * 3. When you copy the framework you must copy this notice with the source
 *    code. You may alter the source code, but you have to put the original
 *    with your altered version.
 * 4. The license is for all files included in this bundle.
 */

/**
 * RSS creation class. A single interface to create news feeds in RSS 2.0
 * (default), 0.91, 0.92 and 0.93 (versions 0.9x and 2.0 are nearly the same),
 * Atom (according to the pre-draft version 0.3), RSS 3.0 (a human readable
 * plain text format according to RFC 822) and ESF (a tabulator separated plain
 * text format).
 *
 * Most constructor and method calls are similar to the RSSBuilder class by
 * Michael Wimmer (http://www.flaimo.com/).
 *
 * Don't hesitate to
 * <a href="http://bugs.maettig.com/">report bugs or feature requests</a>.
 *
 * @author Thiemo Mttig (http://maettig.com/)
 * @version 2004-01-11
 * @package TM
 */
class RSS
{
    /**
     * @var string
     * @access private
     */
    var $encoding = "ISO-8859-1";

    /**
     * @var string
     * @access private
     */
    var $title = "";

    /**
     * @var string
     * @access private
     */
    var $description = "";

    /**
     * @var string Author of the feed.
     * @access private
     */
    var $author = "";

    /**
     * @var string The feeds main language. Defaults to "de-DE".
     * @access private
     */
    var $language = "de-DE";

    /**
     * @var int Unix timestamp the feed changed last. Defaults to the last item date.
     * @access private
     */
    var $date = 0;

    /**
     * @var string
     * @access private
     */
    var $link = "";

    /**
     * @var string
     * @access private
     */
    var $imageUrl = "";

    /**
     * @var array
     * @access private
     */
    var $items = array();

    /**
     * @var int Maximum number of items. Defaults to 15.
     */
    var $maxItemCount = 15;

    /**
     * @var int Maximum item age in days. Defaults to 30.
     */
    var $maxItemAge = 30; // days

    /**
     * @access private
     */
    var $namespaces = array();

    /**
     * @access private
     */
    var $docs = array(
        // 2000-06-04, by Dave Winer.
        '0.91' => "http://backend.userland.com/rss091",
        // 2000-12-09, unused cause there's no docs element in RSS 1.0 or DC.
        '1.0' => "http://web.resource.org/rss/1.0/spec",
        // 2000-12-25, by Dave Winer.
        '0.92' => "http://backend.userland.com/rss092",
        // 2001-04-20, by Dave Winer.
        '0.93' => "http://backend.userland.com/rss093",
        // 2002-08-19, by Dave Winer.
        '2.0' => "http://blogs.law.harvard.edu/tech/rss",
        // 2002-09-06, by Aaron Swartz, unused cause there's no docs element.
        '3.0' => "http://www.aaronsw.com/2002/rss30",
        // 2002-09-24, unused cause there's no docs element in ESF.
        'ESF' => "http://www.aquarionics.com/article/name/esf",
        // 2003-12-12, pre-draft, still in developement.
        'Atom' => "http://www.mnot.net/drafts/draft-nottingham-atom-format-02.html");

    /**
     * @var string
     * @access private
     */
    var $output = "";

    /**
     * Creates a new RSS object.
     *
     * @param encoding string Defaults to "ISO-8859-1".
     * @param link string Leave empty to generate a default value.
     * @param title string
     * @param description string
     * @param imageUrl string
     * @param category string Unused.
     * @param cache int Unused.
     * @return RSS
     */
    function RSS($encoding = "", $link = "", $title = "", $description = "",
        $imageUrl = "", $category = "", $cache = "")
    {
        global $PHP_SELF;
        if (isset($_SERVER['PHP_SELF'])) $PHP_SELF = $_SERVER['PHP_SELF'];

        if ($encoding) $this->encoding = $encoding;
        $this->link = $this->_realpath($link, "http://" . getenv('SERVER_NAME') . $PHP_SELF);
        $this->title = $title;
        $this->description = $description;
        $this->imageUrl = $this->_realpath($imageUrl);
    }

    /**
     * Adds a new item to the feed.
     *
     * @param id mixed Set to true to use the link's value as id.
     * @param title string Leave empty to generate a title out of the description.
     * @param link string Permanent absolute or relative link to the item.
     * @param description string Full content or abstract of the item.
     * @param subject string Unused.
     * @param date int Unix timestamp the item was published.
     * @param author string Unused.
     * @param commentLink mixed Set to true to use the link as comment link too.
     * @param commentCount int Number of comments already posted.
     * @param commentRss string Link to this items comments feed.
     * @return bool
     */
    function addItem($id = "", $title = "", $link = "", $description = "",
        $subject = "", $date = 0, $author = "",
        $commentLink = "", $commentCount = null, $commentRss = "")
    {
        // Don't add too much items.
        if (count($this->items) >= $this->maxItemCount) return false;

        if (! is_numeric($date)) $date = strtotime($date);
	    if ($date < 0) $date = 0;

        // Don't add future items.
        if ($date > time()) return true;
        // Don't add old items, but add at least one item.
        if ($date && $this->items && $date < time() - $this->maxItemAge * 86400)
            return false;

        $link = $this->_realpath($link);
        if ($commentLink === true) $commentLink = $link;
        else $commentLink = $this->_realpath($commentLink);
        $commentRss = $this->_realpath($commentRss);
        if ($id === true) $id = $link;

        $fragments = parse_url($this->link);
        // Make all relative links absolute.
        $description = preg_replace(
            '{<(a\s[^>]*href|img\s[^>]*src)(=["\']?)/}is',
            '<\1\2http://' . $fragments['host'] . '/', $description);
        // Remove any "./" from links and image URLs.
        $description = preg_replace(
            '{<(a\s[^>]*href|img\s[^>]*src)(=["\']?)\./}is',
            '<\1\2', $description);
        $description = preg_replace(
            '{(<a\s[^>]*href=["\']?)#}is',
            '\1' . $this->link . '#', $description);
        $description = preg_replace(
            '{<(a\s[^>]*href|img\s[^>]*src)=("([^:"]*)"|\'([^:\']*)\'|([^:\s>]+))([\s>])}is',
            '<\1="' . dirname($this->link . "file") . '/\3\4\5"\6', $description);

        $title = preg_replace('/\s+/s', ' ', trim($title));
        if (! $title)
        {
            $title = preg_replace('{</?(br|div|p)[^>]*>}is', ' ', $description);
            $title = strip_tags($title);
            // Decode any HTML special chars (&lt;, &amp; etc.).
            $title = strtr($title, array_flip(get_html_translation_table(HTML_ENTITIES)));
            $title = preg_replace('/\s+/s', ' ', trim($title));
            if (! $title)
            {
                if (preg_match('/alt\s*=\s*("([^"]+)"|\'([^\']+)\'|([^\s>]+))/is',
                    $description, $matches))
                    @$title = $matches[2] . $matches[3] . $matches[4];
                elseif (preg_match('/title\s*=\s*("([^"]+)"|\'([^\']+)\'|([^\s>]+))/is',
                    $description, $matches))
                    @$title = $matches[2] . $matches[3] . $matches[4];
            }
            // Use first sentence as title if it's between 10 and 50 characters.
            $title = preg_replace('/(^.{8,48}?[^\d\s.!?][.!?])\s.*$/s', '\1', $title);
            $title = preg_replace('/(^.{8,44}?[^:\s]:\s).*$/s', '\1...', $title);
            // Crop the title if it's longer than 50 characters.
            if (strlen($title) > 50)
                $title = preg_replace('/(^.{1,46}\s).*$/s', '\1...', $title);
            // Use the date as title if it's still empty.
            if (! $title && $date) $title = date("Y-m-d H:i", $date);
        }

        // Register namespaces if required.
	    if (is_numeric($commentCount))
	       $this->namespaces['slash'] = "http://purl.org/rss/1.0/modules/slash/";
        if (! empty($commentRss))
            $this->namespaces['wfw'] = "http://wellformedweb.org/CommentAPI/";

        $this->items[] = array(
            'date' => $date,
            'title' => $title,
            'description' => $description,
            'link' => $link,
            'commentLink' => $commentLink,
            'commentCount' => $commentCount,
            'commentRss' => $commentRss,
            'id' => $id
        );
        if ($date > $this->date) $this->date = $date;
        return true;
    }

    /**
     * Adds some meta data (author, language etc.) to the feed.
     *
     * @param publisher string Unused.
     * @param creator string Name of the author of the feed (no email).
     * @param date int Unix timestamp the feed changed last, defaults to the last item date.
     * @param language string Defaults to "de-DE".
     * @param rights string Unused.
     * @param coverage string Unused.
     * @param contributor string Unused.
     * @return void
     */
	function addDCData($publisher = "", $creator = "", $date = 0,
	   $language = "", $rights = "", $coverage = "", $contributor = "")
	{
	    $this->author = $creator;
	    if (! empty($this->author)) $this->namespaces['dc'] = "http://purl.org/dc/elements/1.1/";

	    if ($language) $this->language = $language;

	    if (! is_numeric($date)) $date = strtotime($date);
	    if ($date > 0) $this->date = (int)$date;
	}

    /**
     * @param period string "hourly", "daily", "weekly", "monthly", "yearly".
     * @param frequency int Every x hours, days and so on.
     * @param base string ISO date.
     * @return void
     * @access private
     */
	function addSYData($period = "", $frequency = 0, $base = "")
	{
	    /* not implemented yet */
	}

    /**
     * Returns the final feed as a string.
     *
     * @param version mixed 2.0 (default), 0.91, 0.92, 0.93, 1.0, "Atom", 3.0, "ESF".
     * @return string
     */
    function getRSSOutput($version = "")
    {
        if (! $this->output) $this->createOutput($version);
        return $this->output;
    }

    /**
     * Prints the final feed to the browser.
     *
     * @param version mixed 2.0 (default), 0.91, 0.92, 0.93, 1.0, "Atom", 3.0, "ESF".
     * @return void
     */
    function outputRSS($version = "")
    {
        if (! $this->output) $this->createOutput($version);
        if ($version == 3.0 || strtoupper($version) == "ESF")
        {
            header("Content-type: text/plain");
            if ($version == 3.0) $filename = "rss30.txt";
            else $filename = "esf.txt";
        }
        else
        {
            header("Content-type: text/xml");
            if (strtolower($version) == "atom") $filename = "atom.xml";
            else $filename = "rss" . str_replace(".", "", $version) . ".xml";
        }
        header("Content-Length: " . strlen($this->output));
        header("Content-Disposition: inline; filename=" . $filename);
        echo $this->output;
    }

    /**
     * @access private
     */
	function createOutput($version = "")
	{
	    // RSS 0.9 is very similar to 1.0, both are modules in the RDF.
        if ($version == 1.0 || $version == 0.9) $this->createOutputV100();
        elseif (stristr($version, "Atom")) $this->createOutputAtom();
        elseif (stristr($version, "Klip")) $this->createOutputKlip();
        elseif (stristr($version, "Food")) $this->createOutputKlipFood();
        elseif ($version == 3.0) $this->createOutputV300();
        elseif (stristr($version, "ESF")) $this->createOutputESF();
        else $this->createOutputV200($version);
	}

    /**
     * RSS 1.0 (and 0.9) template (RDF Site Summary).
     *
     * @access private
     */
    function createOutputV100()
    {
        $this->output = "<?xml version=\"1.0\" encoding=\"" . $this->encoding . "\"?>\n";
        if (! empty($this->css))
            $this->output .= "<?xml-stylesheet type=\"text/css\" href=\"" . $this->css . "\"?>\n";

        $this->output .= "<rdf:RDF xmlns=\"http://purl.org/rss/1.0/\"";
        $this->namespaces['rdf'] = "http://www.w3.org/1999/02/22-rdf-syntax-ns#";
        $this->namespaces['dc'] = "http://purl.org/dc/elements/1.1/";
        foreach ($this->namespaces as $namespace => $url)
            $this->output .= " xmlns:$namespace=\"$url\"";
        $this->output .= ">\n";

        $this->output .= "<channel";
        if (! empty($this->link))
            $this->output .= " rdf:about=\"" . $this->link . "\"";
        $this->output .= ">\n";
        if (! empty($this->title))
            $this->output .= "<title>" . htmlspecialchars($this->title) . "</title>\n";
        if (! empty($this->description))
        {
            $this->output .= "<description>";
            $this->output .= htmlspecialchars($this->description);
            $this->output .= "</description>\n";
        }
        if (! empty($this->author))
            $this->output .= "<dc:creator>" . htmlspecialchars($this->author) . "</dc:creator>\n";
        if (! empty($this->language))
            $this->output .= "<dc:language>" . $this->language . "</dc:language>\n";
        if (! empty($this->date))
        {
            $this->output .= "<dc:date>";
            $this->output .= substr_replace(date("Y-m-d\TH:iO", $this->date), ":", 19, 0);
            $this->output .= "</dc:date>\n";
        }
        if (! empty($this->link))
            $this->output .= "<link>" . $this->link . "</link>\n";
        // <image> is optional.
        if (! empty($this->imageUrl))
            $this->output .= "<image rdf:resource=\"" . $this->mageURL . "\" />\n";
        $this->output .= "<items>\n<rdf:Seq>\n";
        foreach ($this->items as $item)
        {
            if (! empty($item['link']))
                $this->output .= "<rdf:li resource=\"" . $item['link'] . "\" />\n";
        }
        $this->output .= "</rdf:Seq>\n</items>\n</channel>\n\n";
        // <image> is optional.
        if (! empty($this->imageUrl))
        {
            $this->output .= "<image rdf:about=\"" . $this->imageUrl . "\">\n";
            $this->output .= "<url>" . $this->imageUrl . "</url>\n";
            $this->output .= "<title>" . htmlspecialchars($this->title) . "</title>\n";
            $this->output .= "<link>" . $this->link . "</link>\n</image>\n\n";
        }
        foreach ($this->items as $item)
        {
            $this->output .= "<item";
            if (! empty($item['link']))
                $this->output .= " rdf:about=\"" . $item['link'] . "\"";
            $this->output .= ">\n";
            if (! empty($item['date']))
            {
                $this->output .= "<dc:date>";
                $this->output .= substr_replace(date("Y-m-d\TH:iO", $item['date']), ":", 19, 0);
                $this->output .= "</dc:date>\n";
            }
            if (! empty($item['title']))
                $this->output .= "<title>" . htmlspecialchars($item['title']) . "</title>\n";
            if (! empty($item['description']))
            {
                $this->output .= "<description>";
                $this->output .= htmlspecialchars($item['description']);
                $this->output .= "</description>\n";
            }
            if (! empty($item['link']))
                $this->output .= "<link>" . $item['link'] . "</link>\n";
            if (is_numeric($item['commentCount']))
                $this->output .= "<slash:comments>" . $item['commentCount'] . "</slash:comments>\n";
            if (! empty($item['commentRss']))
                $this->output .= "<wfw:commentRss>" . $item['commentRss'] . "</wfw:commentRss>\n";
            if (! empty($item['id']))
                $this->output .= "<dc:identifier>" . htmlspecialchars($item['id']) . "</dc:identifier>\n";
            $this->output .= "</item>\n\n";
        }
        $this->output .= "</rdf:RDF>";
    }

    /**
     * RSS 2.0, 0.91, 0.92 and 0.93 template.
     *
     * - RSS 0.91 (Rich Site Summary). <image> (sub-element of <channel>) is
     *   required but I ignore that. <description> have to be croped to a
     *   maximum of 500 characters.
     * - RSS 0.92 (Rich Site Summary). The main difference is, "all sub-elements
     *   of <item> are optional". Additionaly: <language> is optional,
     *   <description> could be longer than 500 characters.
     * - RSS 0.93 (Rich Site Summary). The only difference is, there is a
     *   <pubDate> for each item.
     * - RSS 2.0 (Really Simple Syndication). Main differences: <guid> added,
     *   <comments> added, support for namespaces.
     *
     * @access private
     */
    function createOutputV200($version = "2.0")
    {
        // 0.9 will treat as 1.0 (see above), 0.94 is identical to 2.0.
        if ($version < 0.91 || $version > 0.93) $version = "2.0";

        $this->output = "<?xml version=\"1.0\" encoding=\"" . $this->encoding . "\"?>\n";
        if (! empty($this->css))
            $this->output .= "<?xml-stylesheet type=\"text/css\" href=\"" . $this->css . "\"?>\n";

        $this->output .= "<rss version=\"" . $version . "\"";
        // Namespaces are optional since 2.0.
        if ($version >= 2.0)
        {
            foreach ($this->namespaces as $namespace => $url)
                $this->output .= " xmlns:$namespace=\"$url\"";
        }
        $this->output .= ">\n";

        $this->output .= "<channel>\n";
        // <title> is required since 0.91.
        $this->output .= "<title>" . htmlspecialchars($this->title) . "</title>\n";
        // <description> is required since 0.91
        $this->output .= "<description>" . htmlspecialchars($this->description) . "</description>\n";
        // Namespace based elements are optional since 2.0.
        if (! empty($this->author) && $version >= 2.0)
            $this->output .= "<dc:creator>" . htmlspecialchars($this->author) . "</dc:creator>\n";
        // <language> is required in 0.91, optional since 0.92.
        if (! empty($this->language) || $version <= 0.91)
            $this->output .= "<language>" . $this->language . "</language>\n";
        // <lastBuildDate> is optional since 0.91.
        if (! empty($this->date))
            $this->output .= "<lastBuildDate>" . date("r", $this->date) . "</lastBuildDate>\n";
        // <link> is required since 0.91.
        $this->output .= "<link>" . $this->link . "</link>\n";
        // <image> is optional since Netscape's 0.9. In UserLand's 0.91 to 0.93
        // it's required. I ignore these rule.
        if (! empty($this->imageUrl))
        {
            // <url>, <title> and <link> are required since 0.91.
            $this->output .= "<image>\n<url>" . $this->imageUrl . "</url>\n";
            $this->output .= "<title>" . htmlspecialchars($this->title) . "</title>\n";
            $this->output .= "<link>" . $this->link . "</link>\n</image>\n";
        }
        // <docs> is optional since 0.91.
        if (! empty($this->docs[$version]))
            $this->output .= "<docs>" . $this->docs[$version] . "</docs>\n";
        // <generator> is optional since 2.0.
        if ($version >= 2.0)
            $this->output .= "<generator>TM::RSS</generator>\n";
        foreach ($this->items as $item)
        {
            $this->output .= "\n<item>\n";
            // <pubDate> is optional since 0.93.
            if (! empty($item['date']) && $version >= 0.93)
                $this->output .= "<pubDate>" . date("r", $item['date']) . "</pubDate>\n";
            // <title> is required in 0.91, optional since 0.92.
            if (! empty($item['title']) || $version <= 0.91)
                $this->output .= "<title>" . htmlspecialchars($item['title']) . "</title>\n";
            // <description> is optional since 0.91.
            if (! empty($item['description']))
            {
                $this->output .= "<description>";
                $this->output .= htmlspecialchars($item['description']);
                $this->output .= "</description>\n";
            }
            // <link> is required in 0.91, optional since 0.92.
            if (! empty($item['link']) || $version <= 0.91)
                $this->output .= "<link>" . $item['link'] . "</link>\n";
            if ($version >= 2.0)
            {
                // <comments> is optional since 2.0.
                if (! empty($item['commentLink']))
                    $this->output .= "<comments>" . $item['commentLink'] . "</comments>\n";
                if (is_numeric($item['commentCount']))
                    $this->output .= "<slash:comments>" . $item['commentCount'] . "</slash:comments>\n";
                if (! empty($item['commentRss']))
                    $this->output .= "<wfw:commentRss>" . $item['commentRss'] . "</wfw:commentRss>\n";
                // <guid> is optional since 2.0.
                if (! empty($item['id']))
                    $this->output .= "<guid>" . htmlspecialchars($item['id']) . "</guid>\n";
            }
            $this->output .= "</item>\n";
        }
        $this->output .= "\n</channel>\n</rss>";
    }

    /**
     * @access private
     */
    function createOutputAtom()
    {
        $this->output = "<?xml version=\"1.0\" encoding=\"" . $this->encoding . "\"?>\n";
        if (! empty($this->css))
            $this->output .= "<?xml-stylesheet type=\"text/css\" href=\"" . $this->css . "\"?>\n";
        $this->output .= "<feed version=\"0.3\" xmlns=\"http://purl.org/atom/ns#\"";
        if (! empty($this->language))
            $this->output .= " xml:lang=\"" . $this->language . "\"";
        $this->output .= ">\n";
        $this->output .= "<title>" . htmlspecialchars($this->title) . "</title>\n";
        if (! empty($this->description))
            $this->output .= "<tagline>" . htmlspecialchars($this->description) . "</tagline>\n";
        $this->output .= "<author>\n";
        $this->output .= "<name>" . htmlspecialchars($this->author) . "</name>\n";
        $this->output .= "</author>\n";
        $this->output .= "<modified>";
        $this->output .= substr_replace(date("Y-m-d\TH:iO", $this->date), ":", 19, 0);
        $this->output .= "</modified>\n";
        $this->output .= "<link rel=\"alternate\" type=\"text/html\"";
        $this->output .= " href=\"" . $this->link . "\"/>\n";
        // <info> is optional in 0.3.
        if (! empty($this->docs['Atom']))
            $this->output .= "<info>" . $this->docs['Atom'] . "</info>\n";
        // <generator> is optional in 0.3.
        $this->output .= "<generator>TM::RSS</generator>\n\n";
        foreach ($this->items as $item)
        {
            $this->output .= "<entry>\n";
            $date = substr_replace(date("Y-m-d\TH:iO", $item['date']), ":", 19, 0);
            $this->output .= "<issued>" . $date . "</issued>\n";
            $this->output .= "<modified>" . $date . "</modified>\n";
            $this->output .= "<title>" . htmlspecialchars($item['title']) . "</title>\n";
            if (! empty($item['description']))
            {
                $this->output .= "<content type=\"text/html\" mode=\"escaped\">";
                $this->output .= htmlspecialchars($item['description']);
                $this->output .= "</content>\n";
            }
            $this->output .= "<link rel=\"alternate\" type=\"text/html\"";
            $this->output .= " href=\"" . $item['link'] . "\"/>\n";
            // <id> is optional in 0.3.
            if (! empty($item['id']))
                $this->output .= "<id>" . htmlspecialchars($item['id']) . "</id>\n";
            $this->output .= "</entry>\n\n";
        }
        $this->output .= "</feed>";
    }

    /**
     * @access private
     */
    function createOutputKlip()
    {
        global $PHP_SELF, $QUERY_STRING;
        if (isset($_SERVER['PHP_SELF'])) $PHP_SELF = $_SERVER['PHP_SELF'];
        if (isset($_SERVER['QUERY_STRING'])) $QUERY_STRING = $_SERVER['QUERY_STRING'];
        $klipLocation = htmlspecialchars($PHP_SELF . "?" . $QUERY_STRING);

        $this->output = "<?xml version=\"1.0\" encoding=\"" . $this->encoding . "\"?>\n";
        $this->output .= "<klip>\n<owner>\n";
        if (! empty($this->author))
            $this->output .= "<author>" . htmlspecialchars($this->author) . "</author>\n";
        $this->output .= "<web>" . htmlspecialchars($this->link) . "</web>\n";
        $this->output .= "</owner>\n<identity>\n";
        if (! empty($this->title))
            $this->output .= "<title>" . htmlspecialchars($this->title) . "</title>\n";
        $this->output .= "<uniqueid>" . htmlspecialchars($this->link) . "</uniqueid>\n";
        if (! empty($this->date))
            $this->output .= "<lastmodified>" . date("Y.m.d:Hi", $this->date) . "</lastmodified>\n";
        if (! empty($this->description))
            $this->output .= "<description>" . htmlspecialchars($this->description) . "</description>\n";
        $this->output .= "</identity>\n<locations>\n";
        $this->output .= "<defaultlink>" . htmlspecialchars($this->link) . "</defaultlink>\n";
        $this->output .= "<contentsource>" . $klipLocation . "&amp;version=food</contentsource>\n";
        if (! empty($this->imageUrl))
            $this->output .= "<banner>" . htmlspecialchars($this->imageUrl) . "</banner>\n";
        $this->output .= "<kliplocation>" . $klipLocation . "</kliplocation>\n";
        $this->output .= "</locations>\n<setup>\n";
        if (strlen($this->language) > 4)
            $this->output .= "<country>" . substr($this->language, 3) . "</country>\n";
        if (strlen($this->language) > 1)
            $this->output .= "<language>" . substr($this->language, 0, 2) . "</language>\n";
        $this->output .= "</setup>\n</klip>";
    }

    /**
     * @access private
     */
    function createOutputKlipFood()
    {
        $this->output = "<?xml version=\"1.0\" encoding=\"" . $this->encoding . "\"?>\n";
        $this->output .= "<klipfood>\n";
        foreach ($this->items as $item)
        {
            $this->output .= "<item";
            // iid= is optional.
            if (! empty($item['id']))
                $this->output .= " iid=\"" . htmlspecialchars($item['id']) . "\"";
            $this->output .= ">\n";
            // <title> is required.
            $this->output .= "<title>" . htmlspecialchars($item['title']) . "</title>\n";
            // <note> is optional.
            if (! empty($item['description']))
            {
                $this->output .= "<note>" . htmlspecialchars($item['description']) . "</note>\n";
            }
            // <link> is required.
            $this->output .= "<link>" . $item['link'] . "</link>\n";
            $this->output .= "</item>\n\n";
        }
        $this->output .= "</klipfood>";
    }

    /**
     * RSS 3.0 template (according to the specification by Aaron Swartz,
     * http://www.aaronsw.com/2002/rss30).
     *
     * @access private
     */
    function createOutputV300()
    {
        if (! empty($this->title))
            $this->output .= "title: " . $this->title . "\r\n";
        if (! empty($this->description))
            $this->output .= "description: " . $this->description . "\r\n";
        if (! empty($this->author))
            $this->output .= "creator: " . $this->author . "\r\n";
        if (! empty($this->language))
            $this->output .= "language: " . $this->language . "\r\n";
        if (! empty($this->date))
        {
            $this->output .= "last-modified: ";
            $this->output .= substr_replace(date("Y-m-d\TH:iO", $this->date), ":", 19, 0);
            $this->output .= "\r\n";
        }
        if (! empty($this->link))
            $this->output .= "link: " . $this->link . "\r\n";
        $this->output .= "generator: TM::RSS\r\n";
        foreach ($this->items as $item)
        {
            $this->output .= "\r\n";
            if (! empty($item['date']))
            {
                $this->output .= "created: ";
                $this->output .= substr_replace(date("Y-m-d\TH:iO", $item['date']), ":", 19, 0);
                $this->output .= "\r\n";
            }
            if (! empty($item['title']))
                $this->output .= "title: " . $item['title'] . "\r\n";
            if (! empty($item['link']))
                $this->output .= "link: " . $item['link'] . "\r\n";
            if (! empty($item['description']))
            {
                // Prepare the plain description text for use in RSS 3.0 only.
                $plain = $item['description'];
                $plain = preg_replace('{<img\s[^>]*alt="([^"]+)"[^>]*>}is', '[Bild: \1]', $plain);
                $plain = preg_replace('{<img\s[^>]*title="([^"]+)"[^>]*>}is', '[Bild: \1]', $plain);
                $plain = preg_replace('{<img\s[^>]*>}is', '[Bild]', $plain);
                $plain = preg_replace('{\s*</?(br|div|p)[^>]*>\s*}', "\r\n ", $plain);
                $this->urlReferences = "";
                $plain = preg_replace_callback(
                    '{<a\s[^>]*href=("([^"]*)"|\'([^\']*)\'|([^\s>]*))[^>]*>(.*?)</a>}is',
                    array(&$this, 'urlCallback'), $plain);
                $plain = trim(strip_tags($plain));
                // Decode any HTML special chars (&lt;, &amp; etc.).
                $plain = strtr($plain, array_flip(get_html_translation_table(HTML_ENTITIES)));

                // Special trick, use "\r\n " as line separator (note the space).
                $this->output .= wordwrap("description: " . $plain, 64, "\r\n ");
                // Add the URL references ([1] etc.). They should not wraped!
                $this->output .= "\r\n" . $this->urlReferences;
            }
        }
    }

    /**
     * Callback function for use in preg_replace_callback(), needed for RSS 3.0
     * only.
     *
     * @access private
     */
    function urlCallback($matches)
    {
        static $n;
        $url = "] " . $matches[2] . $matches[3] . $matches[4] . "\n";
        // Create no new reference if this URL is already in the list.
        if (strpos($this->urlReferences, $url)) return $matches[5];
        // Reset the counter if another item started.
        if (! $this->urlReferences) $n = 0;
        $n++;
        // Collect all links/URIs in the string.
        $this->urlReferences .= " [" . $n . $url;
        // Insert a number (e.g. [4]) to any matched link.
        return $matches[5] . "[" . $n . "]";
    }

    /**
     * @access private
     */
    function createOutputESF()
    {
        if (! empty($this->title))
            $this->output .= "title\t" . $this->title . "\n";
        if (! empty($this->author))
            $this->output .= "contact\t(" . $this->author . ")\n";
        if (! empty($this->link))
            $this->output .= "link\t" . $this->link . "\n";
        $this->output .= "\n";
        foreach ($this->items as $item)
        {
            $this->output .= $item['date'] . "\t";
            $this->output .= $item['title'] . "\t";
            $this->output .= $item['link'] . "\n";
        }
    }

    /**
     * @access private
     */
    function _realpath($url, $base = "")
    {
        // Do not return because <channel><link> can not be empty (first call).
        if (! $url && ! $base) return "";
        // On first call, $base is "http://" . $SERVER_NAME . $PHP_SELF.
        if (! $base) $base = $this->link;
        if (substr($url, 0, 1) == "/")
        {
            $components = parse_url($base);
            return "http://" . $components['host'] . $url;
        }
        if (substr($url, 0, 2) == "./") $url = substr($url, 2);
        if (substr($url, 0, 1) == "#") return $base . $url;
        if (! strpos($url, ":")) return dirname($base . "file") . "/" . $url;
        return $url;
    }
}

?>